<?php

// var_dump($variable);

// В PHP используется динамическая типизация. Язык имеет несколько типов, которые будут рассмотрены ниже.

// 1) Встроенные типы
	// 1.1) NULL
	$variable = null; // NULL

	// 1.2) Скалярные типы
		// 1.2.1) Логические значения (bool, boolean)
		$variable = true; // bool(true)
		$variable = false; // bool(false)

		// 1.2.2) Целые числа (int, integer - двоичные, восьмеричные и шестнадцатеричные числа)
		$variable = 123; // int(123)

		// 1.2.3) Числа с плавающей точкой (float)
		$variable = 123.45; // float(123.45)

		// 1.2.4) Строки (string)
		$variable = 'text'; // string(4) "text"
		$variable = "text"; // string(4) "text"
		$variable = <<<END
			heredoc
END; // string(10) " heredoc"
		$variable = <<<'EOD'
			nowdoc (без обработки переменных, спецсимволов и т.д.)
EOD; // string(94) " nowdoc (без обработки переменных, спецсимволов и т.д.)"

		// 1.2.5) Числовые строки (int или float)
		$variable = '123'; // string(3) "123"
		$variable = '123.45'; // string(6) "123.45"

	// 1.3) Массивы (array)
	$variable = array(1, 2, array(),); // array(3) { [0]=> int(1) [1]=> int(2) [2]=> array(0) { } }
	$variable = [1 => 'test', 2, [],]; // array(3) { [1]=> string(4) "test" [2]=> int(2) [3]=> array(0) { } }

	// 1.4) Объекты (object)
	class A {
		public static function test() {
			return 'test';
		}
	}

	$variable = new A(); // object(A)#1 (0) { }

	// 1.5) Перечисления
	/* enum Suit {
		case One;
		case Two;
	}

	function test_enum(Suit $s) {
		// ...
	}

	$variable = test_enum(Suit::One); */

	// 1.6) Ресурсы (resource)
	$variable = fopen(__DIR__.'/index.php', 'r'); // resource(3) of type (stream)

	// 1.7) Mixed (object, resource, array, string, float, int, bool, null)
	$variable = 'string'; // string(6) "string"
	$variable = 123; // int(123)

	// 1.8) Void
	function test_void(): void {
		// ...
	}

	// 1.9) Never
	function test_never(): never {
		exit();
	}

	// 1.10) Относительные типы классов: self, parent и static
	$variable = new A(); // object(A)#1 (0) { }

// 2) Типы значений
	// 2.1) true
	// function get_always_returns_true(): true {}
	// 2.2) false
	// function get_always_returns_false(): false {}

// 3) Определяемые пользователем типы (класс-типы)
	// 3.1) Интерфейсы (interface)
	// 3.2) Классы (class)
	// 3.3) Перечисления (enum)

// 4) Callable и callback-функции
	function test() {
		return 'test';
	}

	$variable = call_user_func('test'); // string(4) "test"
	$variable = call_user_func(array('A', 'test')); // string(4) "test"

// 5) Итерируемые значения (iterable)
	function test_iterable(iterable $iterable) {
		foreach ($iterable as $value) {
			// ...
		}
	}



//

$variable = 123;
var_dump($variable); // int(123)
$variable = (string)$variable;
var_dump($variable); // string(3) "123"
